#!/usr/bin/python

import gpiozero
import socket
import struct
import time
import traceback

from threading import Thread


class DummyMotor():
    def __init__(self, id):
        self.id = id
        self.speed = 0

    def stop(self):
        if self.speed != 0:
            print(f'{self.id}: STOP')
        self.speed = 0

    def forward(self, speed):
        if self.speed != speed:
            print(f'{self.id}: FWD {speed}')
            self.speed = speed

    def backward(self, speed):
        if self.speed != speed:
            print(f'{self.id}: BAK {speed}')
            self.speed = speed


class Controller(object):
    def __init__(self, port=5001):
        self.port = port
        self.cmd_left = 0
        self.cmd_right = 0
        self.last_command_time = 0
        try:
            self.left_motor = gpiozero.Motor(18, 17)
            self.right_motor = gpiozero.Motor(14, 4)
        except:
            self.left_motor = DummyMotor('L')
            self.right_motor = DummyMotor('R')
        self.running = True
        self.receiver_thread = Thread(target=self.udp_receiver)
        self.receiver_thread.start()
        self.control_thread = Thread(target=self.motor_controller)
        self.control_thread.start()

    def udp_receiver(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.bind(('0.0.0.0', self.port))
        while self.running:
            try:
                data, _ = sock.recvfrom(4)
                self.cmd_left, self.cmd_right = struct.unpack('xxbb', data)
                self.last_command_time = time.time()
            except:
                traceback.print_exc()

    def accept_command(self, left, right):
        self.cmd_left = left * 128
        self.cmd_right = right * 128
        self.last_command_time = time.time()

    def motor_controller(self):
        while self.running:
            try:
                if time.time() - self.last_command_time > 2.0:
                    self.left_motor.stop()
                    self.right_motor.stop()
                else:
                    left = max(min(self.cmd_left / 127.0, 1), -1)
                    right = max(min(self.cmd_right / 127.0, 1), -1)
                    if right == 0:
                        self.left_motor.stop()
                    elif left > 0:
                        self.left_motor.forward(left)
                    else:
                        self.left_motor.backward(-left)
                    if right == 0:
                        self.right_motor.stop()
                    elif right > 0:
                        self.right_motor.forward(right)
                    else:
                        self.right_motor.backward(-right)
            except:
                traceback.print_exc()
            time.sleep(0.05)


if __name__ == '__main__':
    controller = Controller()
    while True:
        print(controller.left_motor.value, controller.right_motor.value)
        time.sleep(1)
