"use strict";

var Touchpad = function(svg) {
    this.svg = svg
    this.x = 0
    this.y = 0
    this.dotSize = 10
    this.dotStyle = 'fill: #000; stroke: #444; stroke-width: 20px;'
    this.indicatorStyle = 'fill: #AAA; stroke: #444; stroke-width: 2px;'
    this.indicatorBackgroundStyle = 'fill: #DDD; stroke: #444; stroke-width: 2px;'
    this.indicatorSize = 0.05
    this.mouseDown = false
    this.dot = null
    this.leftIndicator = null
    this.rightIndicator= null
    this.gamepadIndex = null
    this.forward = 0
    this.right = 0
    this.leftMotor = 0
    this.rightMotor = 0
    this.prevLeftMotor = 0
    this.prevRightMotor = 0
    this.prevCommandSentTimestamp = 0
    this.deadZone = 5.0
    this.controlScale = 1.0 / (128.0 - this.deadZone)
    this.socket = null
    this.accelerationRate = 0.05;

    this.init = function() {
        this.resize()
        this.repaint()
        let mousedown = this.mousedown.bind(this)
        let mousemove = this.mousemove.bind(this)
        let mouseup = this.mouseup.bind(this)
        let touchHandler = this.touchHandler.bind(this)
        this.svg.addEventListener('mousedown', mousedown)
        this.svg.addEventListener('mousein', mousedown)
        this.svg.addEventListener('mouseup', mouseup)
        this.svg.addEventListener('mousemove', mousemove)
        this.svg.addEventListener('touchstart', touchHandler)
        this.svg.addEventListener('touchend', touchHandler)
        this.svg.addEventListener('touchmove', touchHandler)
        window.addEventListener("gamepadconnected", this.gamepadConnected.bind(this))
        window.addEventListener("gamepaddisconnected", this.gamepadDisconnected.bind(this))
//        var socketio = io()
        this.socket = io()
        this.socket.on('connect', this.socketConnected.bind(this))
        setInterval(this.readGamepad.bind(this), 50)
        setInterval(this.sendCommand.bind(this), 50)
    }

    this.resize = function() {
        var rect = this.svg.getBoundingClientRect()  // get the bounding rectangle
        this.width = rect.width
        this.height = rect.height
        this.centerX = this.width / 2.0
        this.centerY = this.height / 2.0
        this.dotSize = Math.min(this.width, this.height) * 0.05
        this.scaleX = (this.centerX - this.dotSize) / 128.0
        this.scaleY = (this.centerY - this.dotSize) / 128.0
    }

    this.repaint = function() {
        SVG.clear(this.svg)
        SVG.rect(this.svg, 0, 0, this.indicatorSize * this.width, this.height,
                           1.0, null, this.indicatorBackgroundStyle)
        SVG.rect(this.svg, this.width - this.indicatorSize * this.width - 1, 0,
                           this.indicatorSize * this.width, this.height,
                           1.0, null, this.indicatorBackgroundStyle)
        this.leftIndicator = SVG.circle(this.svg, this.indicatorSize * this.width, this.centerY,
                                                  this.indicatorSize * 0.8 * this.width, 1.0, this.indicatorStyle)
        this.rightIndicator = SVG.circle(this.svg, this.width - this.indicatorSize * this.width - 1, this.centerY,
                                                   this.indicatorSize * 0.8 * this.width, 1.0, this.indicatorStyle)
        this.dot = SVG.circle(this.svg, this.centerX + this.x * this.scaleX,
                                        this.centerY + this.y * this.scaleY,
                                        this.dotSize, 1.0, this.dotStyle)
    }

    this.mousedown = function(e) {
        this.mouseDown = true
        this.mousemove(e)
    }

    this.mouseup = function(e) {
        this.mouseDown = false
        this.x = 0
        this.y = 0
        this.updateControl(e)
    }

    this.mousemove = function(e) {
        if (!this.mouseDown || !this.dot) return
        this.x = Math.min(Math.max((e.layerX - this.centerX) / this.scaleX, -128), 127)
        this.y = Math.min(Math.max((e.layerY - this.centerY) / this.scaleY, -128), 127)
        this.updateControl()
    }

    this.gamepadConnected = function(e) {
        this.gamepadIndex = e.gamepad.index
    }

    this.gamepadDisconnected = function(t, e) {
        this.gamepadIndex = null
    }

    this.readGamepad = function() {
        if (this.gamepadIndex == null) return
        var axes = navigator.getGamepads()[this.gamepadIndex].axes
        this.x = axes[0] * 128
        this.y = - axes[1] * 128
        this.updateControl()
    }

    this.updateControl = function() {
        if (Math.abs(this.x) < this.deadZone && Math.abs(this.y) < this.deadZone) {
            this.forward = 0.0
            this.right = 0.0
            this.leftMotor = 0.0
            this.rightMotor = 0.0
        } else {
            if (this.y < 0) {
                this.forward = (-this.y - this.deadZone) * this.controlScale
            } else {
                this.forward = (-this.y + this.deadZone) * this.controlScale
            }
            this.right = this.x * this.controlScale
            this.leftMotor = Math.max(Math.min(this.forward + this.right, 1.0), -1.0)
            this.rightMotor = Math.max(Math.min(this.forward - this.right, 1.0), -1.0)
        }
        this.dot.setAttribute('cx', this.centerX + this.x * this.scaleX)
        this.dot.setAttribute('cy', this.centerY + this.y * this.scaleY)
        this.leftIndicator.setAttribute('cy', this.centerY - this.leftMotor * this.centerY)
        this.rightIndicator.setAttribute('cy', this.centerY - this.rightMotor * this.centerY)
    }

    this.socketConnected = function() {
        this.socket.emit('messages', {data: 'Controller connected!'});
    }

    this.touchHandler = function(event) {
        var touches = event.changedTouches, first = touches[0], type = "";
        switch(event.type) {
            case "touchstart": type = "mousedown"; break;
            case "touchmove":  type = "mousemove"; break;
            case "touchend":   type = "mouseup";   break;
            default:           return;
        }
        var simulatedEvent = document.createEvent("MouseEvent");
        simulatedEvent.initMouseEvent(type, true, true, window, 1,
                                      first.screenX, first.screenY,
                                      first.clientX, first.clientY, false,
                                      false, false, false, 0/*left*/, null);
        first.target.dispatchEvent(simulatedEvent);
        event.preventDefault();
    }

    this.sendCommand = function() {
        if (this.leftMotor == this.prevLeftMotor &&
            this.rightMotor == this.prevRightMotor &&
            Date.now() - this.prevCommandSentTimestamp < 500)
        {
            return
        }
        this.prevLeftMotor = this.accelerate(this.leftMotor, this.prevLeftMotor);
        this.prevRightMotor = this.accelerate(this.rightMotor, this.prevRightMotor);
//        console.log(this.prevLeftMotor, this.prevRightMotor);
        this.socket.emit('control', {L: this.prevLeftMotor, R: this.prevRightMotor})
//        this.prevLeftMotor = this.leftMotor
//        this.prevRightMotor = this.rightMotor
        this.prevCommandSentTimestamp = Date.now()
    }

    this.accelerate = function(target, current) {
        if (target > current) {
            let acc = Math.max(this.accelerationRate, this.accelerationRate * (target - current));
//            console.log(acc);
            return current + Math.min(target - current, acc);
        } else {
            let acc = Math.max(this.accelerationRate, this.accelerationRate * (current - target));
//            console.log(acc);
            return current - Math.min(current - target, acc);
        }
    }

    this.init()
    return this
}